#include "stdafx.h"
#include "../index.h"
#include "Effect.h"

namespace nsDX9A{
namespace nsFramework{
namespace nsEffect{

//RXgN^
cEffecterFadeIn::cEffecterFadeIn(DX9AGraphic* GM){
	m_GM = GM;
	m_Color = 0;
	m_Time = 0;
	m_IsActive = false;
}

//p[^ݒ
void cEffecterFadeIn::SetParam(COLORREF Color, DWORD Time){
	m_Color = Color;
	m_Time = Time;
}

//GtFNgJn
void cEffecterFadeIn::Start(){
	m_Timer.Reset();
	m_IsActive = true;
}
//GtFNgI
void cEffecterFadeIn::End(){
	m_IsActive = false;
}

//`
void cEffecterFadeIn::OnDraw(){
	if(m_IsActive){
		m_Timer.Flush();
		int FadeRate = 255 - (255 * m_Timer.Get() / m_Time);
		if(FadeRate<0){
			FadeRate = 0;
			m_IsActive = false;
		}
		RECT ScreenRect;
		ScreenRect.left = 0;
		ScreenRect.top = 0;
		ScreenRect.right = m_GM->GetScreenWidth();
		ScreenRect.bottom = m_GM->GetScreenHeight();
		m_GM->DrawRect(ScreenRect, D3DCOLOR_ARGB(static_cast<BYTE>(FadeRate), GetRValue(m_Color), GetGValue(m_Color), GetBValue(m_Color)));
	}
}

//GtFNgH
bool cEffecterFadeIn::IsActive() const{
	return(m_IsActive);
}

//|[Y
void cEffecterFadeIn::Pause(){
	m_Timer.Pause();
}
//|[Y
void cEffecterFadeIn::Restart(){
	m_Timer.Restart();
}
//|[YԂH
bool cEffecterFadeIn::IsPause() const{
	return(m_Timer.IsPause());
}



//RXgN^
cEffecterFadeOut::cEffecterFadeOut(DX9AGraphic* GM){
	m_GM = GM;
	m_Color = 0;
	m_Time = 0;
	m_IsActive = false;
}

//p[^ݒ
void cEffecterFadeOut::SetParam(COLORREF Color, DWORD Time){
	m_Color = Color;
	m_Time = Time;
}

//GtFNgJn
void cEffecterFadeOut::Start(){
	m_Timer.Reset();
	m_IsActive = true;
}
//GtFNgI
void cEffecterFadeOut::End(){
	m_IsActive = false;
}

//`
void cEffecterFadeOut::OnDraw(){
	if(m_IsActive){
		m_Timer.Flush();
		int FadeRate = 255 * m_Timer.Get() / m_Time;
		if(FadeRate>255){
			FadeRate = 255;
			m_IsActive = false;
		}
		RECT ScreenRect;
		ScreenRect.left = 0;
		ScreenRect.top = 0;
		ScreenRect.right = m_GM->GetScreenWidth();
		ScreenRect.bottom = m_GM->GetScreenHeight();
		m_GM->DrawRect(ScreenRect, D3DCOLOR_ARGB(static_cast<BYTE>(FadeRate), GetRValue(m_Color), GetGValue(m_Color), GetBValue(m_Color)));
	}
}

//GtFNgH
bool cEffecterFadeOut::IsActive() const{
	return(m_IsActive);
}

//|[Y
void cEffecterFadeOut::Pause(){
	m_Timer.Pause();
}
//|[Y
void cEffecterFadeOut::Restart(){
	m_Timer.Restart();
}
//|[YԂH
bool cEffecterFadeOut::IsPause() const{
	return(m_Timer.IsPause());
}



//RXgN^
cEffecterAnimation::cEffecterAnimation(DX9AGraphic* GM, DX9AParam* PM, IDraw2DHelper* Helper) : m_EffectSprite(GM, PM, Helper) {
	m_X = m_Y = 0;
	m_IsCenter = false;
	m_NowScaling = 1.0f;
	m_Degree = 0.0f;
	m_EffectSprite.SetUpdateType(1);
	End();
}

//p[^ݒiڎwj
void cEffecterAnimation::SetParam(const vector< pair<string, RECT> >& AnimPattern, float BaseScaling, DWORD WaitTime){
	m_EffectSprite.SetParam(AnimPattern, BaseScaling, WaitTime);
}
//p[^ݒiݒt@Cwj
void cEffecterAnimation::SetParam(const string& INIFileName){
	m_EffectSprite.SetParam(INIFileName);
}
//\Wݒ
void cEffecterAnimation::SetPosition(int X, int Y, bool IsCenter){
	m_X = X;
	m_Y = Y;
	m_IsCenter = IsCenter;
}
//݂̊gkݒ
void  cEffecterAnimation::SetNowScaling(float NowScaling){
	m_NowScaling = NowScaling;
}
//]pxݒ
void cEffecterAnimation::SetDegree(float Degree){
	m_Degree = Degree;
}

//GtFNgJn
void cEffecterAnimation::Start(){
	m_EffectSprite.SetAnimationNo(0);
}
//GtFNgI
void cEffecterAnimation::End(){
	m_EffectSprite.SetAnimationNo(UINT_MAX);
}
//`
void cEffecterAnimation::OnDraw(){
	if(IsActive()){
		m_EffectSprite.OnDraw(m_X, m_Y, m_IsCenter, m_NowScaling, m_Degree);
		m_EffectSprite.Update();
	}
}
//GtFNgH
bool cEffecterAnimation::IsActive() const{
	return(!m_EffectSprite.IsEnd());
}

//|[Y
void cEffecterAnimation::Pause(){
	m_EffectSprite.Pause();
}
//|[Y
void cEffecterAnimation::Restart(){
	m_EffectSprite.Restart();
}
//|[YԂH
bool cEffecterAnimation::IsPause() const{
	return(m_EffectSprite.IsPause());
}



//RXgN^
CEffectProcess::CEffectProcess(DX9AProcess* Manager, int ClassID) : CProcess(Manager, ClassID) {
	m_Effect = NULL;
	m_IsStartEffect = false;
}
//fXgN^
CEffectProcess::~CEffectProcess(){
	SAFE_DELETE(m_Effect);
}

//
void CEffectProcess::Release(){
	SAFE_DELETE(m_Effect);
	CProcess::Release();
}

//
void CEffectProcess::Init(){
	m_IsStartEffect = false;
}

//vZXsiOj
void CEffectProcess::OnBegin(){
	if(!m_IsStartEffect){
		m_Effect->Start();
		m_IsStartEffect = true;
	}
}
//vZXsi`j
void CEffectProcess::OnDraw(){
	m_Effect->OnDraw();
}
//vZXsi㏈j
void CEffectProcess::OnEnd(){
	if(m_IsStartEffect && !m_Effect->IsActive()){
		if(GetNext() && GetParent()){
			multimap<string, IProcess*> Temp = GetParent()->GetChildren();
			for(multimap<string, IProcess*>::iterator it=Temp.begin();it!=Temp.end();it++){
				if(it->second==this){
					SwitchToNext(it->first, true, true);
				}
			}
		}
		Release();
	}
}

//|[Y
void CEffectProcess::Pause(){
	m_Effect->Pause();
}
//|[Y
void CEffectProcess::Restart(){
	m_Effect->Restart();
}
//|[YԂH
bool CEffectProcess::IsPause() const{
	return(m_Effect->IsPause());
}

//GtFNgݒ
void CEffectProcess::SetEffect(IEffecter* Effect){
	SAFE_DELETE(m_Effect);
	m_Effect = Effect;
}

}
}
}