#include "stdafx.h"
#include "../index.h"
#include "FilePack.h"

namespace nsDX9A{
namespace nsFramework{
namespace nsFilePack{

//RXgN^it@CpbNǗNXAt@CpbNwj
CFilePackReader::CFilePackReader(DX9AFilePack* Manager, const string& FilePackName) : m_FileDatas(Manager->m_FilePacks.find(FilePackName)->second) {
	ZeroMemory(&m_FileData, sizeof(m_FileData));
	if(Manager->m_FilePacks.find(FilePackName)==Manager->m_FilePacks.end()){
		DX9AErrOut("Failed : CFilePackReader::CFilePackReader Invalid FilePackName\n");
		WARNING
	}
	m_FilePackName = FilePackName;
}
//RXgN^it@CpbNǗNXAt@CpbNAt@CA[hwj
CFilePackReader::CFilePackReader(DX9AFilePack* Manager, const string& FilePackName, const char* FileName, const char* Mode) : m_FileDatas(Manager->m_FilePacks.find(FilePackName)->second) {
	ZeroMemory(&m_FileData, sizeof(m_FileData));
	if(Manager->m_FilePacks.find(FilePackName)==Manager->m_FilePacks.end()){
		DX9AErrOut("Failed : CFilePackReader::CFilePackReader Invalid FilePackName\n");
		WARNING
	}
	m_FilePackName = FilePackName;
	fopen(FileName, Mode);
}

//fopen
bool CFilePackReader::fopen(const char* FileName, const char* Mode){
	if(stream){
		DX9AErrOut("Failed : CFilePackReader::fopen Open Data Already Exists\n");
		return(false);
	}
	if(m_FileDatas.FileData.find(FileName)==m_FileDatas.FileData.end()){
		DX9AErrOut("Failed : CFilePackReader::fopen File Not Found\n");
		return(false);
	}
	if(Mode){
		size_t szMode = strlen(Mode);
		if(szMode<1 && Mode[0]!='r'){
			DX9AErrOut("Failed : CFilePackReader::fopen Invalid File Mode\n");
			return(false);
		}
		if(szMode<2 && Mode[1]=='+'){
			DX9AErrOut("Failed : CFilePackReader::fopen Invalid File Mode\n");
			return(false);
		}
	}
	stream = ::fopen(m_FilePackName.c_str(), Mode);
	if(stream){
		m_FileData = m_FileDatas.FileData.find(FileName)->second;
		fseek(0, SEEK_SET);
	}
	return(stream!=NULL);
}
//fsize
long CFilePackReader::fsize(){
	return(m_FileData.FileSize);
}

//fgetc
bool CFilePackReader::fgetc(char* c){
	if(feof()){
		return(false);
	}
	return(CFileReader::fgetc(c));
}
//fgets
bool CFilePackReader::fgets(char* s, int n){
	if(feof()){
		return(false);
	}
	for(int i=0;i<n-1;i++){
		char c;
		fgetc(&c);
		s[i] = c;
		if(c=='\0'){
			return(true);
		}
		if(c=='\n'){
			s[i] = '\0';
			return(true);
		}
		if(feof()){
			s[i+1] = '\0';
			return(true);
		}
	}
	s[n-1] = '\0';
	return(true);
}
//fread
size_t CFilePackReader::fread(void* buffer, size_t size, size_t count){
	if(feof()){
		return(0);
	}
	if(m_FileData.FileSize-ftell()>=static_cast<long>(size*count)){
		return(CFileReader::fread(buffer, size, count));
	}
	for(size_t i=0;i<count;i++){
		if(m_FileData.FileSize-ftell()<static_cast<long>(size)){
			return(i);
		}
		else{
			CFileReader::fread(reinterpret_cast<BYTE*>(buffer)+(size*i), size, 1);
		}
	}
	return(count);
}

//ftell
long CFilePackReader::ftell(){
	return(CFileReader::ftell()-static_cast<long>(m_FileData.FilePos));
}
//fseek
bool CFilePackReader::fseek(long offset, int origin){
	switch(origin){
		case(SEEK_SET):
			if(offset<0 || offset>m_FileData.FileSize){
				return(false);
			}
			return(CFileReader::fseek(static_cast<long>(m_FileData.FilePos) + offset, SEEK_SET));
		case(SEEK_CUR):
			if( CFileReader::ftell()+offset<m_FileData.FilePos || 
				CFileReader::ftell()+offset>m_FileData.FilePos+m_FileData.FileSize){
				return(false);
			}
			return(CFileReader::fseek(offset, SEEK_CUR));
		case(SEEK_END):
			if(offset>0 || offset<-m_FileData.FileSize){
				return(false);
			}
			return(CFileReader::fseek(static_cast<long>(m_FileData.FilePos) + m_FileData.FileSize - offset, SEEK_SET));
		default:
			return(false);
	}
}
//fgetpos
bool CFilePackReader::fgetpos(fpos_t* pos){
	if(CFileReader::fgetpos(pos)){
		*pos -= m_FileData.FilePos;
		return(true);
	}
	return(false);
}
//fsetpos
bool CFilePackReader::fsetpos(const fpos_t* pos){
	if(*pos>m_FileData.FileSize){
		return(false);
	}
	return(CFileReader::fsetpos(m_FileData.FilePos+pos));
}

//feof
bool CFilePackReader::feof(){
	return(CFileReader::feof() || CFileReader::ftell()>m_FileData.FilePos+m_FileData.FileSize);
}



//RXgN^it@C̓NXAt@Co̓NXwj
DX9AFilePack::DX9AFilePack(IFileReader* FileReader, IFileWriter* FileWriter){
	m_CryptKey = 0;
	m_FileReader = FileReader;
	m_FileWriter = FileWriter;
}
//fXgN^
DX9AFilePack::~DX9AFilePack(){
	SAFE_DELETE(m_FileReader);
	SAFE_DELETE(m_FileWriter);
}

//ÍL[ݒ
void DX9AFilePack::SetCryptKey(BYTE Key){
	m_CryptKey = Key;
}
//t@C̓NXݒ
void DX9AFilePack::SetFileReader(IFileReader* FileReader){
	SAFE_DELETE(m_FileReader);
	m_FileReader = FileReader;
}
//t@Co̓NXݒ
void DX9AFilePack::SetFileWriter(IFileWriter* FileWriter){
	SAFE_DELETE(m_FileWriter);
	m_FileWriter = FileWriter;
}

//t@CpbNJ
bool DX9AFilePack::OpenFilePack(const string& FilePackName){
	if(!m_FileReader){
		DX9AErrOut("Failed : DX9AFilePack::OpenFilePack DX9AFilePack Not Initialized\n");
		return(false);
	}
	if(FilePackName==""){
		return(false);
	}
	if(m_FilePacks.find(FilePackName)!=m_FilePacks.end()){
		return(true);
	}
	if(m_FileReader->fopen(FilePackName.c_str(), "rb")){
		char Header[8] = {0};
		m_FileReader->fread(Header, sizeof(Header), 1);
		if(	Header[0]=='.' && Header[1]=='D' && Header[2]=='X' && Header[3]=='9' && 
			Header[4]=='A' && Header[5]=='p' && Header[6]=='a' && Header[7]=='k' ){
			m_FilePacks.insert(pair<string, FilePackHeader>(FilePackName, FilePackHeader()));
			unsigned int FileNum;
			m_FileReader->fread(&FileNum, sizeof(FileNum), 1);
			for(unsigned int i=0;i<FileNum;i++){
				string FileName;
				LoadCryptedString(m_FileReader, m_CryptKey, &FileName);
				m_FilePacks.find(FilePackName)->second.FileData.insert(pair<string, PackedFileData>(FileName, PackedFileData()));
				m_FileReader->fread(&(m_FilePacks.find(FilePackName)->second.FileData.find(FileName)->second.FilePos), sizeof(fpos_t), 1);
				m_FileReader->fread(&(m_FilePacks.find(FilePackName)->second.FileData.find(FileName)->second.FileSize), sizeof(long), 1);
			}
			return(m_FileReader->fclose());
		}
		else{
			m_FileReader->fclose();
			return(false);
		}
	}
	return(false);
}
//t@CpbN
void DX9AFilePack::CloseFilePack(const string& FilePackName){
	m_FilePacks.erase(m_FilePacks.find(FilePackName));
}
//ΏۂpbLO
bool DX9AFilePack::ConvertToFilePack(const string& TargetDirectory, DX9AParam& DX9AP){
	//֐pNX
	class InnerFunction{
	public:
		static void GetFileList(const string& TargetDirectory, map<string, PackedFileData>& PackedFileNameList, map<string, string>& PackedFileFullPathList){
			string TopDirectoryName = "";
			for(int i=static_cast<int>(TargetDirectory.size());i>=0;i--){
				if(TargetDirectory[i]=='\\' || TargetDirectory[i]=='/'){
					TopDirectoryName = TargetDirectory.substr(i+1);
					break;
				}
			}
			if(TopDirectoryName==""){
				TopDirectoryName = TargetDirectory;
			}
			_GetFileList(TargetDirectory, TopDirectoryName, PackedFileNameList, PackedFileFullPathList);
		}
	protected:
		static void _GetFileList(const string& TargetDirectory, const string& TopDirectoryName, map<string, PackedFileData>& PackedFileNameList, map<string, string>& PackedFileFullPathList){
			HANDLE hr;
			WIN32_FIND_DATA fd;
			char SearchFileName[_MAX_PATH] = {0};
			sprintf(SearchFileName, "%s/*.*", TargetDirectory.c_str());
			hr = FindFirstFile(SearchFileName, &fd);
			if(hr!=INVALID_HANDLE_VALUE){
				do{
					if(strcmp(fd.cFileName, ".") && strcmp(fd.cFileName, "..")){
						char PackedFileName[_MAX_PATH] = {0};
						sprintf(PackedFileName, "%s/%s", TopDirectoryName.c_str(), fd.cFileName);
						char PackedFileFullPath[_MAX_PATH] = {0};
						sprintf(PackedFileFullPath, "%s/%s", TargetDirectory.c_str(), fd.cFileName);
						if(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY){
							_GetFileList(PackedFileFullPath, PackedFileName, PackedFileNameList, PackedFileFullPathList);
						}
						else{
							PackedFileData FileData;
							FileData.FileSize = (fd.nFileSizeHigh * MAXDWORD) + fd.nFileSizeLow;
							PackedFileNameList.insert(pair<string, PackedFileData>(PackedFileName, FileData));
							PackedFileFullPathList.insert(pair<string, string>(PackedFileName, PackedFileFullPath));
						}
					}
				}while(FindNextFile(hr, &fd));
				FindClose(hr);
			}
		}
	};
	if(!m_FileWriter || !m_FileReader){
		DX9AErrOut("Failed : DX9AFilePack::ConvertToFilePack DX9AFilePack Not Initialized\n");
		return(false);
	}
	if(!(GetFileAttributes(TargetDirectory.c_str()) & FILE_ATTRIBUTE_DIRECTORY)){
		return(false);
	}
	//t@Cꗗ擾
	FilePackHeader HeaderData;
	map<string, string> PackedFileFullPathList;
	InnerFunction::GetFileList(TargetDirectory, HeaderData.FileData, PackedFileFullPathList);
	//inidat̕ϊs
	for(map<string, string>::iterator it=PackedFileFullPathList.begin();it!=PackedFileFullPathList.end();it++){
		char ExtName[_MAX_EXT];
		_splitpath(it->second.c_str(), NULL, NULL, NULL, ExtName);
		if(strcmp(ExtName, ".ini")==0){
			if(DX9AP.LoadParamINIFile(it->second, true)){
				DX9AP.SaveParamDATFile(it->second, it->first);
			}
			else{
				if(!DX9AP.LoadParamDATFile(it->second, true)){
					DX9AErrOut("Failed : DX9AFilePack::ConvertToFilePack ParameterFile Convert Failed\n");
					return(false);
				}
			}
		}
	}
	//ēxt@Cꗗ擾
	HeaderData.FileData.clear();
	PackedFileFullPathList.clear();
	InnerFunction::GetFileList(TargetDirectory, HeaderData.FileData, PackedFileFullPathList);
	//wb_̃TCYvZ
	size_t HeaderSize = 8 + sizeof(unsigned int);
	for(map<string, PackedFileData>::iterator it=HeaderData.FileData.begin();it!=HeaderData.FileData.end();it++){
		HeaderSize += (strlen(it->first.c_str()) + 1) + sizeof(fpos_t) + sizeof(long);
	}
	//t@C̈ʒuvZ
	fpos_t FilePos = HeaderSize;
	for(map<string, PackedFileData>::iterator it=HeaderData.FileData.begin();it!=HeaderData.FileData.end();it++){
		it->second.FilePos = FilePos;
		FilePos += it->second.FileSize;
	}
	//Ԃɏ݂Jn
	string TargetFileName = TargetDirectory + ".dat";
	if(m_FileWriter->fopen(TargetFileName.c_str(), "wb")){
		char Header[8] = { '.', 'D', 'X', '9', 'A', 'p', 'a', 'k' };
		m_FileWriter->fwrite(Header, sizeof(Header), 1);
		unsigned int FileNum = static_cast<unsigned int>(HeaderData.FileData.size());
		m_FileWriter->fwrite(&FileNum, sizeof(FileNum), 1);
		for(map<string, PackedFileData>::iterator it=HeaderData.FileData.begin();it!=HeaderData.FileData.end();it++){
			SaveCryptedString(m_FileWriter, m_CryptKey, it->first);
			m_FileWriter->fwrite(&(it->second.FilePos), sizeof(fpos_t), 1);
			m_FileWriter->fwrite(&(it->second.FileSize), sizeof(long), 1);
		}
		for(map<string, string>::iterator it=PackedFileFullPathList.begin();it!=PackedFileFullPathList.end();it++){
			if(m_FileReader->fopen(it->second.c_str(), "rb")){
				BYTE* FileByteData = NULL;
				long FileByteSize = m_FileReader->fsize();
				FileByteData = new BYTE[FileByteSize];
				m_FileReader->fread(FileByteData, FileByteSize, 1);
				m_FileReader->fclose();
				m_FileWriter->fwrite(FileByteData, FileByteSize, 1);
				delete(FileByteData);
				m_FileReader->fclose();
			}
			else{
				DX9AErrOut("Failed : DX9AFilePack::ConvertToFilePack File Can't Open\n");
				return(false);
			}
		}
		return(m_FileWriter->fclose());
	}
	return(false);
}

}
}
}