#include "stdafx.h"
#include "../index.h"
#include "Param.h"

namespace nsDX9A{
namespace nsFramework{
namespace nsParam{

//RXgN^it@CAp[^̓NX̐U镑wj
ParamReader::ParamReader(DX9AParam* Manager, const string& FileName, int BPR){
	m_Manager = Manager;
	m_FileName = FileName;
	m_BPR = BPR;
	if(m_BPR & BPR_LOAD_ONCREATE){
		Load((m_BPR & BPR_RELOAD_ONCREATE)!=0);
	}
}

//fXgN^
ParamReader::~ParamReader(){
	if(m_BPR & BPR_UNLOAD_ONDESTROY){
		Unload();
	}
}

//p[^ij擾
int ParamReader::GetIntParam(const string& SectionName, const string& ParamName, int DefaultValue){
	return(m_Manager->GetIntParam(m_FileName, SectionName, ParamName, DefaultValue));
}
//p[^ij擾
string ParamReader::GetStrParam(const string& SectionName, const string& ParamName, const string& DefaultValue){
	return(m_Manager->GetStrParam(m_FileName, SectionName, ParamName, DefaultValue));
}

//p[^t@C[h
bool ParamReader::Load(bool AllowReload){
	return(m_Manager->LoadParamDATFile(m_FileName, AllowReload) || m_Manager->LoadParamINIFile(m_FileName, AllowReload));
}
//p[^t@CA[h
void ParamReader::Unload(){
	m_Manager->Unload(m_FileName);
}



//RXgN^it@CAp[^o̓NX̐U镑wj
ParamWriter::ParamWriter(DX9AParam* Manager, const string& FileName, int BPW){
	m_Manager = Manager;
	m_FileName = FileName;
	m_AutoSaveFileName = FileName;
	m_BPW = BPW;
	if(m_BPW & BPW_LOAD_ONCREATE){
		Load((m_BPW & BPW_RELOAD_ONCREATE)!=0);
	}
}

//fXgN^
ParamWriter::~ParamWriter(){
	if(m_BPW & BPW_SAVEINI_ONDESTROY){
		SaveINI(m_AutoSaveFileName);
	}
	if(m_BPW & BPW_SAVEDAT_ONDESTROY){
		SaveDAT(m_AutoSaveFileName);
	}
	if(m_BPW & BPW_UNLOAD_ONDESTROY){
		Unload();
	}
}

//p[^ijݒ
void ParamWriter::SetIntParam(const string& SectionName, const string& ParamName, int ParamValue){
	m_Manager->SetIntParam(m_FileName, SectionName, ParamName, ParamValue);
}
//p[^ijݒ
void ParamWriter::SetStrParam(const string& SectionName, const string& ParamName, const string& ParamValue){
	m_Manager->SetStrParam(m_FileName, SectionName, ParamName, ParamValue);
}

//p[^t@Ciinit@C`jZ[u
void ParamWriter::SaveINI(const string& DestFileName){
	m_Manager->SaveParamINIFile(m_FileName, DestFileName);
}
//p[^t@Cidatt@C`jZ[u
void ParamWriter::SaveDAT(const string& DestFileName){
	m_Manager->SaveParamDATFile(m_FileName, DestFileName);
}
//p[^t@C[h
bool ParamWriter::Load(bool AllowReload){
	return(m_Manager->LoadParamDATFile(m_FileName, AllowReload) || m_Manager->LoadParamINIFile(m_FileName, AllowReload));
}
//p[^t@CA[h
void ParamWriter::Unload(){
	m_Manager->Unload(m_FileName);
}

//Z[ũt@Cݒ
void ParamWriter::SetAutoSaveDestFileName(const string& FileName){
	m_AutoSaveFileName = FileName;
}



//RXgN^it@C̓NXAt@Co̓NXwj
DX9AParam::DX9AParam(IFileReader* FileReader, IFileWriter* FileWriter){
	m_FileReader = FileReader;
	m_FileWriter = FileWriter;
	m_CryptKey = 0;
}
//fXgN^
DX9AParam::~DX9AParam(){
	SAFE_DELETE(m_FileReader);
	SAFE_DELETE(m_FileWriter);
}

//p[^ij擾
int DX9AParam::GetIntParam(const string& FileName, const string& SectionName, const string& ParamName, int DefaultValue){
	//֐pNX
	class InnerFunction{
	public:
		//擾p[^H
		static bool IsNumString(const string& Target, int& Result){
			if(Target==""){
				return(false);
			}
			Result = 0;
			char Temp[64 * 1024] = {0};
			strcpy(Temp, Target.c_str());
			for(unsigned int i=0;Temp[i]!='\0';i++){
				if(Temp[i]>='0' && Temp[i]<='9'){
					Result = (Result * 10) + (Temp[i] - '0');
				}
				else{
					if(i!=0 || Temp[i]!='-'){
						return(false);
					}
				}
			}
			if(Temp[0]=='-'){
				Result *= -1;
			}
			return(true);
		}
	};
	//1.datt@C`init@C`ł̃[h݂遨Ń[hsȂDefaultValueԂ
	if(!LoadParamDATFile(FileName, false) && !LoadParamINIFile(FileName, false)){
		return(DefaultValue);
	}
	//2.p[^̎擾݂
	string ParamValue;
	if(GetParam(FileName, SectionName, ParamName, ParamValue)){
		int ReturnValue;
		if(InnerFunction::IsNumString(ParamValue, ReturnValue)){
			return(ReturnValue);
		}
	}
	//3.p[^̎擾s\߁ApHĂ̎擾݂iċAĂяoj
	return(m_ParamFiles.find(FileName)->second.RootFile=="" ? DefaultValue : GetIntParam(m_ParamFiles.find(FileName)->second.RootFile, SectionName, ParamName, DefaultValue));
}
//p[^ij擾
string DX9AParam::GetStrParam(const string& FileName, const string& SectionName, const string& ParamName, const string& DefaultValue){
	//֐pNX
	class InnerFunction{
	public:
		//擾p[^񂩁H
		static bool IsStrString(const string& Target, string& Result){
			if(Target==""){
				return(false);
			}
			Result = "";
			char Temp[64 * 1024] = {0};
			strcpy(Temp, Target.c_str());
			if(Temp[0]!='\"' || Temp[strlen(Temp)-1]!='\"'){
				return(false);
			}
			for(unsigned int i=1;i<strlen(Temp)-1;i++){
				Result += Temp[i];
			}
			return(true);
		}
	};
	//1.datt@C`init@C`ł̃[h݂遨Ń[hsȂDefaultValueԂ
	if(!LoadParamDATFile(FileName, false) && !LoadParamINIFile(FileName, false)){
		return(DefaultValue);
	}
	//2.p[^̎擾݂
	string ParamValue;
	if(GetParam(FileName, SectionName, ParamName, ParamValue)){
		string ReturnValue;
		if(InnerFunction::IsStrString(ParamValue, ReturnValue)){
			return(ReturnValue);
		}
	}
	//3.p[^̎擾s\߁ApHĂ̎擾݂iċAĂяoj
	return(m_ParamFiles.find(FileName)->second.RootFile=="" ? DefaultValue : GetStrParam(m_ParamFiles.find(FileName)->second.RootFile, SectionName, ParamName, DefaultValue));
}
//p[^ijݒ
void DX9AParam::SetIntParam(const string& FileName, const string& SectionName, const string& ParamName, int ParamValue){
	char ProcessedParamValue[64 * 1024] = {0};
	sprintf(ProcessedParamValue, "%d", ParamValue);
	if(m_ParamFiles.find(FileName)==m_ParamFiles.end()){
		m_ParamFiles.insert(pair<string, Param>(FileName, Param()));
	}
	if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)==m_ParamFiles.find(FileName)->second.Params.end()){
		m_ParamFiles.find(FileName)->second.Params.insert(pair<string, map<string, string> >(SectionName, map<string, string>()));
	}
	if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)==m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.end()){
		m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.insert(pair<string, string>(ParamName, ProcessedParamValue));
	}
	else{
		m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)->second = ProcessedParamValue;
	}
}
//p[^ijݒ
void DX9AParam::SetStrParam(const string& FileName, const string& SectionName, const string& ParamName, const string& ParamValue){
	char ProcessedParamValue[64 * 1024] = {0};
	sprintf(ProcessedParamValue, "\"%s\"", ParamValue.c_str());
	if(m_ParamFiles.find(FileName)==m_ParamFiles.end()){
		m_ParamFiles.insert(pair<string, Param>(FileName, Param()));
	}
	if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)==m_ParamFiles.find(FileName)->second.Params.end()){
		m_ParamFiles.find(FileName)->second.Params.insert(pair<string, map<string, string> >(SectionName, map<string, string>()));
	}
	if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)==m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.end()){
		m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.insert(pair<string, string>(ParamName, ProcessedParamValue));
	}
	else{
		m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)->second = ProcessedParamValue;
	}
}

//ÍL[ݒ
void DX9AParam::SetCryptKey(BYTE Key){
	m_CryptKey = Key;
}
//t@C̓NXݒ
void DX9AParam::SetFileReader(IFileReader* FileReader){
	SAFE_DELETE(m_FileReader);
	m_FileReader = FileReader;
}
//t@Co̓NXݒ
void DX9AParam::SetFileWriter(IFileWriter* FileWriter){
	SAFE_DELETE(m_FileWriter);
	m_FileWriter = FileWriter;
}
//p[^t@Ciinit@C`jZ[u
void DX9AParam::SaveParamINIFile(const string& SrcFileName, const string& DestFileName){
	if(m_ParamFiles.find(SrcFileName)==m_ParamFiles.end()){
		DX9AErrOut("Failed : DX9AParam::SaveParamINIFile SrcFileName Not Exists(%s)\n", SrcFileName.c_str());
		return;
	}
	if(DestFileName==""){
		DX9AErrOut("Failed : DX9AParam::SaveParamINIFile DestFileName Is Empty\n");
		return;
	}
	FILE* stream;
	stream = fopen(DestFileName.c_str(), "wt");
	if(stream){
		fclose(stream);
		char DestFileFullPath[64 * 1024] = {0};
		_fullpath(DestFileFullPath, DestFileName.c_str(), sizeof(DestFileFullPath));
		for(map<string, map<string, string> >::iterator it=m_ParamFiles.find(SrcFileName)->second.Params.begin();it!=m_ParamFiles.find(SrcFileName)->second.Params.end();it++){
			for(map<string, string>::iterator it2=it->second.begin();it2!=it->second.end();it2++){
				WritePrivateProfileString(it->first.c_str(), it2->first.c_str(), it2->second.c_str(), DestFileFullPath);
			}
		}
	}
}
//p[^t@Cidatt@C`jZ[u
void DX9AParam::SaveParamDATFile(const string& SrcFileName, const string& DestFileName){
	if(!m_FileWriter){
		DX9AErrOut("Failed : DX9AParam::SaveParamDATFile DX9AParam Not Initialized\n");
		return;
	}
	if(m_ParamFiles.find(SrcFileName)==m_ParamFiles.end()){
		DX9AErrOut("Failed : DX9AParam::SaveParamDATFile SrcFileName Not Exists(%s)\n", SrcFileName.c_str());
		return;
	}
	if(DestFileName==""){
		DX9AErrOut("Failed : DX9AParam::SaveParamDATFile DestFileName Is Empty\n");
		return;
	}
	if(m_FileWriter->fopen(DestFileName.c_str(), "wb")){
		char Header[8] = { '.', 'I', 'N', 'I', 'c', 'D', 'A', 'T' };
		m_FileWriter->fwrite(Header, sizeof(Header), 1);
		SaveCryptedString(m_FileWriter, m_CryptKey, m_ParamFiles.find(SrcFileName)->second.RootFile);
		unsigned int SectionNum = static_cast<unsigned int>(m_ParamFiles.find(SrcFileName)->second.Params.size());
		m_FileWriter->fwrite(&SectionNum, sizeof(SectionNum), 1);
		for(map<string, map<string, string> >::iterator it=m_ParamFiles.find(SrcFileName)->second.Params.begin();it!=m_ParamFiles.find(SrcFileName)->second.Params.end();it++){
			SaveCryptedString(m_FileWriter, m_CryptKey, it->first);
			unsigned int ParamNum = static_cast<unsigned int>(it->second.size());
			m_FileWriter->fwrite(&ParamNum, sizeof(ParamNum), 1);
			for(map<string, string>::iterator it2=it->second.begin();it2!=it->second.end();it2++){
				SaveCryptedString(m_FileWriter, m_CryptKey, it2->first);
				SaveCryptedString(m_FileWriter, m_CryptKey, it2->second);
			}
		}
		m_FileWriter->fclose();
	}
}
//p[^t@Ciinit@C`j[h
bool DX9AParam::LoadParamINIFile(const string& FileName, bool AllowReload){
	if(FileName==""){
		DX9AErrOut("Failed : DX9AParam::LoadParamINIFile FileName Is Empty\n");
		return(false);
	}
	if(AllowReload){
		Unload(FileName);
	}
	if(m_ParamFiles.find(FileName)!=m_ParamFiles.end()){
		return(true);
	}
	FILE* stream;
	stream = fopen(FileName.c_str(), "rt");
	if(stream){
		fclose(stream);
		char FileFullPath[64 * 1024] = {0};
		_fullpath(FileFullPath, FileName.c_str(), sizeof(FileFullPath));
		m_ParamFiles.insert(pair<string, Param>(FileName, Param()));
		//ZNV擾
		DWORD SectionNamesSize = 0;
		char SectionNames[64 * 1024] = {0};
		SectionNamesSize = GetPrivateProfileSectionNames(SectionNames+1, sizeof(SectionNames)-1, FileFullPath);
		if(SectionNamesSize>0){
			for(DWORD i=0;i<SectionNamesSize;i++){
				if(SectionNames[i]=='\0'){
					m_ParamFiles.find(FileName)->second.Params.insert(pair<string, map<string, string> >(SectionNames+i+1, map<string, string>()));
					//p[^Ep[^l擾
					DWORD ParameterNamesValuesSize = 0;
					char ParameterNamesValues[64 * 1024] = {0};
					ParameterNamesValuesSize = GetPrivateProfileSection(SectionNames+i+1, ParameterNamesValues+1, sizeof(ParameterNamesValues)-1, FileFullPath);
					if(ParameterNamesValuesSize>0){
						for(DWORD j=0;j<ParameterNamesValuesSize;j++){
							if(ParameterNamesValues[j]=='='){
								ParameterNamesValues[j] = '\0';
							}
						}
						string ParamName;
						string ParamValue;
						for(DWORD j=0;j<ParameterNamesValuesSize;j++){
							if(ParameterNamesValues[j]=='\0'){
								if(ParamName.empty()){
									ParamName = ParameterNamesValues + j + 1;
								}
								else{
									ParamValue = ParameterNamesValues + j + 1;
									m_ParamFiles.find(FileName)->second.Params.find(SectionNames+i+1)->second.insert(pair<string, string>(ParamName, ParamValue));
									ParamName.clear();
									ParamValue.clear();
								}
							}
						}
					}
				}
			}
			//p擾
			if(m_ParamFiles.find(FileName)->second.Params.find("Root")!=m_ParamFiles.find(FileName)->second.Params.end()){
				if(m_ParamFiles.find(FileName)->second.Params.find("Root")->second.find("RootFile")!=m_ParamFiles.find(FileName)->second.Params.find("Root")->second.end()){
					m_ParamFiles.find(FileName)->second.RootFile = m_ParamFiles.find(FileName)->second.Params.find("Root")->second.find("RootFile")->second;
					ReplaceString(m_ParamFiles.find(FileName)->second.RootFile, "\"", "");
				}
			}
		}
		return(true);
	}
	DX9AErrOut("Failed : DX9AParam::LoadParamINIFile File Not Found(%s)\n", FileName.c_str());
	return(false);
}
//p[^t@Cidatt@C`j[h
bool DX9AParam::LoadParamDATFile(const string& FileName, bool AllowReload){
	if(!m_FileReader){
		DX9AErrOut("Failed : DX9AParam::LoadParamDATFile DX9AParam Not Initialized\n");
		return(false);
	}
	if(FileName==""){
		DX9AErrOut("Failed : DX9AParam::LoadParamDATFile FileName Is Empty\n");
		return(false);
	}
	if(AllowReload){
		Unload(FileName);
	}
	if(m_ParamFiles.find(FileName)!=m_ParamFiles.end()){
		return(true);
	}
	if(m_FileReader->fopen(FileName.c_str(), "rb")){
		char Header[8] = {0};
		m_FileReader->fread(Header, sizeof(Header), 1);
		if(	Header[0]=='.' && Header[1]=='I' && Header[2]=='N' && Header[3]=='I' && 
			Header[4]=='c' && Header[5]=='D' && Header[6]=='A' && Header[7]=='T' ){
			m_ParamFiles.insert(pair<string, Param>(FileName, Param()));
			LoadCryptedString(m_FileReader, m_CryptKey, &(m_ParamFiles.find(FileName)->second.RootFile));
			unsigned int SectionNum;
			m_FileReader->fread(&SectionNum, sizeof(SectionNum), 1);
			for(unsigned int i=0;i<SectionNum;i++){
				string SectionName;
				LoadCryptedString(m_FileReader, m_CryptKey, &SectionName);
				m_ParamFiles.find(FileName)->second.Params.insert(pair<string, map<string, string> >(SectionName, map<string, string>()));
				unsigned int ParamNum;
				m_FileReader->fread(&ParamNum, sizeof(ParamNum), 1);
				for(unsigned int j=0;j<ParamNum;j++){
					string ParamName;
					string ParamValue;
					LoadCryptedString(m_FileReader, m_CryptKey, &ParamName);
					LoadCryptedString(m_FileReader, m_CryptKey, &ParamValue);
					m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.insert(pair<string, string>(ParamName, ParamValue));
				}
			}
			return(m_FileReader->fclose());
		}
		else{
			m_FileReader->fclose();
			DX9AErrOut("Failed : DX9AParam::LoadParamDATFile File Not INIcDAT\n");
			return(false);
		}
	}
	DX9AErrOut("Failed : DX9AParam::LoadParamDATFile File Not Found(%s)\n", FileName.c_str());
	return(false);
}
//p[^t@CA[h
void DX9AParam::Unload(const string& FileName){
	if(m_ParamFiles.find(FileName)!=m_ParamFiles.end()){
		m_ParamFiles.erase(m_ParamFiles.find(FileName));
	}
}

//p[^擾iʁj
bool DX9AParam::GetParam(const string& FileName, const string& SectionName, const string& ParamName, string& ParamValue){
	if(m_ParamFiles.find(FileName)!=m_ParamFiles.end()){
		if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)!=m_ParamFiles.find(FileName)->second.Params.end()){
			if(m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)!=m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.end()){
				ParamValue = m_ParamFiles.find(FileName)->second.Params.find(SectionName)->second.find(ParamName)->second;
				return(true);
			}
		}
	}
	return(false);
}

}
}
}