#include "stdafx.h"
#include "../index.h"
#include "DX9AInput.h"

namespace nsDX9A{
namespace nsInput{
namespace nsDX9AInput{

//ftHgRXgN^
DX9AInput::DX9AInput(){

	//oϐ
	m_hWnd = NULL;

	ZeroMemory(m_Key.Key, sizeof(m_Key.Key));

	m_MousePos.x = m_MousePos.y = 0;
	m_MouseWheel = 0;
	ZeroMemory(m_MouseButton, sizeof(m_MouseButton));

	m_JoyNum = 0;
	m_AllowX = 0; m_AllowY = 0;

	m_pDI = NULL;
	m_Keyboard = NULL;
	m_Mouse = NULL;

}

//fXgN^
DX9AInput::~DX9AInput(){

	//L[{[h
	if(m_Keyboard){
		m_Keyboard->Unacquire();
		SAFE_RELEASE(m_Keyboard);
	}

	//}EX
	if(m_Mouse){
		m_Mouse->Unacquire();
		SAFE_RELEASE(m_Mouse);
	}

	//WCXeBbN
	for(vector<LPDIRECTINPUTDEVICE8>::iterator it=m_Joystick.begin();it!=m_Joystick.end();it++){
		if((*it)){
			(*it)->Unacquire();
			SAFE_RELEASE((*it));
		}
	}

	SAFE_RELEASE(m_pDI);

}

//iKĂяoƁj
bool DX9AInput::Init(HINSTANCE hInst, HWND hWnd){

	if(m_hWnd){
		DX9AErrOut("Failed : DX9AInput::Init Already Initialized\n");
		return(false);
	}
	m_hWnd = hWnd;

	//DirectInput
	if(FAILED(DirectInput8Create(hInst, DIRECTINPUT_VERSION, IID_IDirectInput8, reinterpret_cast<LPVOID*>(&m_pDI), NULL))){
		DX9AErrOut("Failed : DX9AInput::Init DirectInput8Create\n");
		return(false);
	}

	//L[{[h
	if(FAILED(m_pDI->CreateDevice(GUID_SysKeyboard, &m_Keyboard, NULL))){
		DX9AErrOut("Failed : DX9AInput::Init CreateDevice Keyboard\n");
		return(false);
	}
	if(m_Keyboard){
		if(FAILED(m_Keyboard->SetDataFormat(&c_dfDIKeyboard))){
			DX9AErrOut("Failed : DX9AInput::Init SetDataFormat Keyboard\n");
			return(false);
		}
		if(FAILED(m_Keyboard->SetCooperativeLevel(m_hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE))){
			DX9AErrOut("Failed : DX9AInput::Init SetCooperativeLevel Keyboard\n");
			return(false);
		}
		m_Keyboard->Acquire();
	}

	//}EX
	if(FAILED(m_pDI->CreateDevice(GUID_SysMouse, &m_Mouse, NULL))){
		DX9AErrOut("Failed : DX9AInput::Init CreateDevice Mouse\n");
		return(false);
	}
	if(m_Mouse){
		if(FAILED(m_Mouse->SetDataFormat(&c_dfDIMouse2))){
			DX9AErrOut("Failed : DX9AInput::Init SetDataFormat Mouse\n");
			return(false);
		}
		if(FAILED(m_Mouse->SetCooperativeLevel(m_hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE))){
			DX9AErrOut("Failed : DX9AInput::Init SetCooperativeLevel Mouse\n");
			return(false);
		}
		m_Mouse->Acquire();
	}

	//WCXeBbN
	if(FAILED(m_pDI->EnumDevices(DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback, reinterpret_cast<LPVOID>(this), DIEDFL_ATTACHEDONLY))){
		DX9AErrOut("Failed : DX9AInput::Init EnumDevices Joystick\n");
		return(false);
	}
	if(m_JoyNum>0){
		Joystick Temp;
		ZeroMemory(&Temp, sizeof(Temp));
		for(unsigned int i=0;i<m_JoyNum;i++){
			m_JoyButton.push_back(Temp);
			if(FAILED(m_Joystick[i]->SetDataFormat(&c_dfDIJoystick))){
				DX9AErrOut("Failed : DX9AInput::Init SetDataFormat Joystick\n");
				return(false);
			}
			if(FAILED(m_Joystick[i]->SetCooperativeLevel(m_hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE))){
				DX9AErrOut("Failed : DX9AInput::Init SetCooperativeLevel Joystick\n");
				return(false);
			}
			if(FAILED(m_Joystick[i]->EnumObjects(EnumObjectsCallback, reinterpret_cast<LPVOID>(m_Joystick[i]), DIDFT_ALL))){
				DX9AErrOut("Failed : DX9AInput::Init EnumObjects Joystick\n");
				return(false);
			}
			m_Joystick[i]->Acquire();
		}
	}

	return(true);

}

//͏XV
void DX9AInput::Flush(bool IsFullScreen){

	HRESULT hr;

	//L[{[h
	if(m_Keyboard){
		BYTE DIKS[256];
		hr = m_Keyboard->GetDeviceState(sizeof(DIKS), DIKS);
		if(FAILED(hr)){
			hr = m_Keyboard->Acquire();
			while(hr==DIERR_INPUTLOST){
				hr = m_Keyboard->Acquire();
			}
		}
		else{
			for(unsigned int i=0;i<KEY_MAX;i++){
				m_Key.Key[i] = (DIKS[i] & 0x80) ? true : false;
			}
		}
	}

	//}EX
	if(m_Mouse){
		DIMOUSESTATE2 DIMS;
		hr = m_Mouse->GetDeviceState(sizeof(DIMS), &DIMS);
		if(FAILED(hr)){
			hr = m_Mouse->Acquire();
			while(hr==DIERR_INPUTLOST){
				hr = m_Mouse->Acquire();
			}
		}
		else{
			GetCursorPos(&m_MousePos);
			if(!IsFullScreen){
				ScreenToClient(m_hWnd, &m_MousePos);
			}
			m_MouseWheel = DIMS.lZ;
			for(unsigned int i=0;i<MOUSE_BUTTON_MAX;i++){
				m_MouseButton[i] = (DIMS.rgbButtons[i] & 0x80) ? true : false;
			}
		}
	}

	//WCXeBbN
	for(unsigned int i=0;i<m_JoyNum;i++){
		DIJOYSTATE DIJS;
		hr = m_Joystick[i]->Poll();
		if(FAILED(hr)){
			hr = m_Joystick[i]->Acquire();
			while(hr==DIERR_INPUTLOST){
				hr = m_Joystick[i]->Acquire();
			}
		}
		else{
			ZeroMemory(&(m_JoyButton[i]), sizeof(m_JoyButton[i]));
			if(SUCCEEDED(m_Joystick[i]->GetDeviceState(sizeof(DIJS), &DIJS))){
				m_JoyButton[i].X = (abs(DIJS.lX)<abs(m_AllowX)) ? 0 : DIJS.lX;
				m_JoyButton[i].Y = (abs(DIJS.lY)<abs(m_AllowY)) ? 0 : DIJS.lY;
				m_JoyButton[i].Up = (m_JoyButton[i].Y<0) ? true : false;
				m_JoyButton[i].Down = (m_JoyButton[i].Y>0) ? true : false;
				m_JoyButton[i].Left = (m_JoyButton[i].X<0) ? true : false;
				m_JoyButton[i].Right = (m_JoyButton[i].X>0) ? true : false;
				for(unsigned int j=0;j<JOYSTICK_BUTTON_MAX;j++){
					m_JoyButton[i].Button[j] = (DIJS.rgbButtons[j] & 0x80) ? true : false;
				}
			}
		}
	}

}

//L[擾(1)
KeyBoard DX9AInput::GetKey() const{
	return(m_Key);
}
//L[擾(2)
bool DX9AInput::GetKey(BYTE Code) const{
	return((Code<KEY_MAX) ? m_Key.Key[Code] : false);
}
	
//ʒuݒ
void DX9AInput::SetMousePos(POINT Pos){
	ClientToScreen(m_hWnd, &Pos);
	SetCursorPos(Pos.x, Pos.y);
	m_MousePos = Pos;
}
//ʒu擾
void DX9AInput::GetMousePos(POINT& Pos) const{
	Pos = m_MousePos;
}
//zC[擾
int DX9AInput::GetMouseWheel() const{
	return(m_MouseWheel);
}
//{^擾
bool DX9AInput::GetMouseButton(BYTE Code) const{
	return((Code<MOUSE_BUTTON_MAX) ? m_MouseButton[Code] : false);
}

//擾
DWORD DX9AInput::GetJoyNum() const{
	return(m_JoyNum);
}
//͏擾
void DX9AInput::GetJoystick(Joystick& Data, DWORD ID) const{
	if(ID<m_JoyNum){
		Data = m_JoyButton[ID];
	}
	else{
		DX9AErrOut("Failed : DX9AInput::GetJoystick ID(%d)>=m_JoyNum(%d)\n", ID, m_JoyNum);
		//S0Ȃ̂nĂiLDMj
		Joystick NullObject = {0};
		Data = NullObject;
	}
}
//Vтݒ
void DX9AInput::SetAllow(int AllowX, int AllowY){
	m_AllowX = AllowX;
	m_AllowY = AllowY;
}

//R[obN֐
BOOL CALLBACK DX9AInput::EnumObjectsCallback(LPCDIDEVICEOBJECTINSTANCE DDOI, LPVOID Ref){

	DIPROPRANGE DIPR;
	ZeroMemory(&DIPR, sizeof(DIPR));
	DIPR.diph.dwSize = sizeof(DIPR);
	DIPR.diph.dwHeaderSize = sizeof(DIPR.diph);
	DIPR.diph.dwObj = DDOI->dwType;
	DIPR.diph.dwHow = DIPH_BYID;
	DIPR.lMin = -1000;
	DIPR.lMax = +1000;

	LPDIRECTINPUTDEVICE8 JoystickAccessor = reinterpret_cast<LPDIRECTINPUTDEVICE8>(Ref);
	if(FAILED(JoystickAccessor->SetProperty(DIPROP_RANGE, &(DIPR.diph)))){
		return(DIENUM_STOP);
	}
	else{
		return(DIENUM_CONTINUE);
	}

}
BOOL CALLBACK DX9AInput::EnumJoysticksCallback(const DIDEVICEINSTANCE* DDI, LPVOID Context){

	DWORD JoyNumMax = joyGetNumDevs();
	DX9AInput* DX9AInputAccessor = reinterpret_cast<DX9AInput*>(Context);

	DX9AInputAccessor->m_DevCaps.dwSize = sizeof(DX9AInputAccessor->m_DevCaps);
	DX9AInputAccessor->m_Joystick.push_back(NULL);
	DX9AInputAccessor->m_pDI->CreateDevice(DDI->guidInstance, &(DX9AInputAccessor->m_Joystick.back()), NULL);
	DX9AInputAccessor->m_Joystick.back()->GetCapabilities(&(DX9AInputAccessor->m_DevCaps));

	if(++(DX9AInputAccessor->m_JoyNum)==JoyNumMax){
		return(DIENUM_STOP);
	}
	else{
		return(DIENUM_CONTINUE);
	}

}



//ftHgRXgN^
DX9AInputAccessor::DX9AInputAccessor(){
	Outer = NULL;
}

//eGWݒ
void DX9AInputAccessor::SetParent(DX9AInput* Parent){
	Outer = Parent;
}

//DirectInput8IuWFNg擾
LPDIRECTINPUT8 DX9AInputAccessor::GetDirectInput8(){
	return(Outer->m_pDI);
}



//ANZbT
static DX9AInputAccessor DX9ASingletonDX9AInputAccessor;
//ANZbT擾
DX9AInputAccessor* GetDX9AInputAccessor(DX9AInput* Parent){
	DX9ASingletonDX9AInputAccessor.SetParent(Parent);
	return(&DX9ASingletonDX9AInputAccessor);
}

}
}
}