#include "stdafx.h"
#include "../index.h"
#include "ISoundReader.h"

namespace nsDX9A{
namespace nsSound{
namespace nsISoundReader{

//RXgN^it@Cwj
CWavReader::CWavReader(IFileReader* FileReader, const string& FileName){
	m_FileReader = FileReader;
	if(FileName!=""){
		Open(FileName);
	}
}
//fXgN^iÓIȊ蓖Ă֎~j
CWavReader::~CWavReader(){
	Close();
	SAFE_DELETE(m_FileReader);
}

//
void CWavReader::Release(){
	delete(this);
}

//I[v
bool CWavReader::Open(const string& FileName){
	if(m_FileReader->fisopen()){
		DX9AErrOut("Failed : CWavReader::Open Data Already Exists\n");
		return(false);
	}
	//t@CI[v
	if(!m_FileReader->fopen(FileName.c_str(), "rb")){
		DX9AErrOut("Failed : CWavReader::Open File Not Found\n");
		return(false);
	}
	//RIFFwb_ǂݍ
	DWORD RIFFHead[5];
	m_FileReader->fread(RIFFHead, sizeof(RIFFHead), 1);
	if(RIFFHead[0]!='R'+('I'<<8)+('F'<<16)+('F'<<24) ||
	   RIFFHead[2]!='W'+('A'<<8)+('V'<<16)+('E'<<24) ||
	   RIFFHead[3]!='f'+('m'<<8)+('t'<<16)+(' '<<24)){
		m_FileReader->fclose();
		DX9AErrOut("Failed : CWavReader::Open Unknown Format\n");
		return(false);
	}
	//PCMWAVEFORMATwb_ǂݍ
	m_FileReader->fread(&m_PCMWF, sizeof(m_PCMWF), 1);
	if(m_PCMWF.wf.wFormatTag!=WAVE_FORMAT_PCM){
		m_FileReader->fclose();
		DX9AErrOut("Failed : CWavReader::Open Unknown Format\n");
		return(false);
	}
	//data`Nwb_ǂݎ
	DWORD dataHead[2];
	DWORD Ptr = sizeof(RIFFHead) + RIFFHead[4];
	for(;;){
		if(Ptr>=RIFFHead[1]+8){
			m_FileReader->fclose();
			DX9AErrOut("Failed : CWavReader::Open Unknown Format\n");
			return(false);
		}
		//`Nwb_ǂݍ
		m_FileReader->fseek(Ptr, SEEK_SET);
		m_FileReader->fread(dataHead, sizeof(dataHead), 1);
		if(dataHead[0]=='d'+('a'<<8)+('t'<<16)+('a'<<24)){
			break;
		}
		Ptr += dataHead[1] + 8;
	}
	Ptr += 8;
	m_TopPtr = Ptr;
	m_TotalSize = dataHead[1];
	m_RemainSize = dataHead[1];
	SetLoopIn(0);
	SetLoopOut(GetTotal());
	SetLoopCount(0);
	return(true);
}
//N[Y
void CWavReader::Close(){
	if(m_FileReader->fisopen()){
		m_FileReader->fclose();
	}
}
//I[vH
bool CWavReader::IsOpen() const{
	return(m_FileReader->fisopen());
}

//ǂݍ
bool CWavReader::Read(void* Dest, unsigned long Size, unsigned long* ReadSize){
	if(!IsOpen()){
		DX9AErrOut("Failed : CWavReader::Read Not Open\n");
		return(false);
	}
	const DWORD ReadSize2 = (m_RemainSize < Size) ? m_RemainSize : Size;
	DWORD ActualRead = 0;
	ActualRead = static_cast<DWORD>(m_FileReader->fread(Dest, 1, ReadSize2));
	m_RemainSize -= ActualRead;
	if(ReadSize){
		*ReadSize = ActualRead;
	}
	return(true);
}
//ǂݍ݈ʒuݒ
bool CWavReader::Seek(unsigned long Sample){
	if(!IsOpen()){
		DX9AErrOut("Failed : CWavReader::Seek Not Open\n");
		return(false);
	}
	//V[NvZ
	const DWORD MoveTo = Sample * m_PCMWF.wf.nBlockAlign;
	if(MoveTo>m_TotalSize){
		//Sample傫
		DX9AErrOut("Failed : CWavReader::Seek Sample Too Large\n");
		return(false);
	}
	//V[N
	if(!m_FileReader->fseek(m_TopPtr+MoveTo, SEEK_SET)){
		//V[NłȂ
		DX9AErrOut("Failed : CWavReader::Seek Failed\n");
		return(false);
	}
	//cTCY𒲐
	m_RemainSize = m_TotalSize - MoveTo;
	return(true);
}
//ǂݍ݈ʒu擾
unsigned long CWavReader::Tell() const{
	return((m_TotalSize-m_RemainSize)/m_PCMWF.wf.nBlockAlign);
}
//ǂݍ݈ʒuI[H
bool CWavReader::IsTerminated() const{
	if(IsOpen()){
		if(m_RemainSize==0){
			return(true);
		}
	}
	return(false);
}

//tH[}bg擾
PCMWAVEFORMAT CWavReader::GetFormat() const{
	return(m_PCMWF);
}
//Ŝ̃Tv擾
unsigned long CWavReader::GetTotal() const{
	return(m_TotalSize/m_PCMWF.wf.nBlockAlign);
}
//[vC̃Tv擾
unsigned long CWavReader::GetLoopIn() const{
	return(m_LoopIn);
}
//[vAEg̃Tv擾
unsigned long CWavReader::GetLoopOut() const{
	return(m_LoopOut);
}
//[v񐔂擾
int CWavReader::GetLoopCount() const{
	return(m_LoopCount);
}

//[vC̃Tvݒ
void CWavReader::SetLoopIn(unsigned long Sample){
	m_LoopIn = Sample;
}
//[vAEg̃Tvݒ
void CWavReader::SetLoopOut(unsigned long Sample){
	m_LoopOut = Sample;
}
//[v񐔂ݒ
void CWavReader::SetLoopCount(int LoopCount){
	m_LoopCount = LoopCount;
}



#ifdef USE_OGG

//RXgN^it@C̓NXwj
COggFile::COggFile(IFileReader* FileReader){
	m_FileReader = FileReader;
	m_Offset = 0;
	m_Size = 0;
}
//RXgN^it@C̓NXAt@Cwj
COggFile::COggFile(IFileReader* FileReader, DWORD Offset, DWORD Size){
	m_FileReader = FileReader;
	m_Offset = Offset;
	m_Size = Size;
}

//t@C̓NX擾
IFileReader* COggFile::GetReader() const{
	return(m_FileReader);
}
//ItZbg擾
DWORD COggFile::GetOffset() const{
	return(m_Offset);
}
//TCY擾
DWORD COggFile::GetSize() const{
	return(m_Size);
}
//I[vH
bool COggFile::IsOpen() const{
	return(m_FileReader->fisopen());
}
//I[v
bool COggFile::Open(const string& FileName){
	if(!m_FileReader->fopen(FileName.c_str(), "rb")){
		DX9AErrOut("Failed : COggFile::Open File Not Found\n");
		return(false);
	}
	m_Offset = 0;
	m_Size = m_FileReader->fsize();
	return(true);
}
//N[Y
void COggFile::Close(){
	CallbackClose(this);
}

//R[obN֐
ov_callbacks COggFile::OggCallbacks = {
	COggFile::CallbackRead,
	COggFile::CallbackSeek,
	COggFile::CallbackClose,
	COggFile::CallbackTell
};
size_t COggFile::CallbackRead(void* ptr, size_t size, size_t nmemb, void* pThis){
	COggFile* File = reinterpret_cast<COggFile*>(pThis);
	size_t ActualRead = 0;
	ActualRead = File->GetReader()->fread(ptr, 1, static_cast<DWORD>(size * nmemb));
	return(ActualRead);
}
int COggFile::CallbackSeek(void* pThis, ogg_int64_t offset, int whence){
	COggFile* File = reinterpret_cast<COggFile*>(pThis);
	DWORD MoveMethod;
	DWORD To;
	if(whence==SEEK_CUR){
		MoveMethod = SEEK_CUR;
		To = static_cast<DWORD>(offset);
	}
	else if(whence==SEEK_END){
		MoveMethod = SEEK_SET;
		To = static_cast<DWORD>(offset) + File->GetOffset() + File->GetSize();
	}
	else if(whence==SEEK_SET){
		MoveMethod = SEEK_SET;
		To = static_cast<DWORD>(offset) + File->GetOffset();
	}
	else{
		return(-1);
	}
	return(File->GetReader()->fseek(To, MoveMethod) ? 0 : -1);
}
int COggFile::CallbackClose(void* pThis){
	COggFile* File = reinterpret_cast<COggFile*>(pThis);
	bool Result = File->GetReader()->fclose();
	*File = COggFile(File->GetReader());
	return(Result ? 0 : EOF);
}
long COggFile::CallbackTell(void* pThis){
	COggFile* File = reinterpret_cast<COggFile*>(pThis);
	DWORD Offset = File->GetReader()->ftell();
	return(Offset-File->GetOffset());
}



//RXgN^it@C̓NXAt@Cwj
COggReader::COggReader(IFileReader* FileReader, const string& FileName) : m_File(FileReader) {
	m_FileReader = FileReader;
	m_TotalSamples = 0;
	m_Terminated = false;
	m_LoopIn = 0;
	m_LoopOut = 0;
	m_LoopCount = 0;
	if(FileName!=""){
		Open(FileName);
	}
}
//fXgN^iÓIȊ蓖Ă֎~j
COggReader::~COggReader(){
	Close();
	SAFE_DELETE(m_FileReader);
}

//
void COggReader::Release(){
	delete(this);
}

//I[v
bool COggReader::Open(const string& FileName){
	if(IsOpen()){
		DX9AErrOut("Failed : COggReader::Open Data Already Exists\n");
		return(false);
	}
	//t@CJ
	COggFile File(m_FileReader);
	if(!File.Open(FileName)){
		DX9AErrOut("Failed : COggReader::Open File Not Found(%s)\n", FileName.c_str());
		return(false);
	}
	return(Open(File));
}
//N[Y
void COggReader::Close(){
	if(m_File.IsOpen()){
		ov_clear(&m_VF);
	}
}
//I[vH
bool COggReader::IsOpen() const{
	return(m_File.IsOpen());
}

//ǂݍ
bool COggReader::Read(void* Dest, unsigned long Size, unsigned long* ReadSize){
	if(!IsOpen()){
		DX9AErrOut("Failed : COggReader::Read Not Open\n");
		return(false);
	}
	char* DestPtr = reinterpret_cast<char*>(Dest);
	DWORD Remain = Size;
	while(Remain>0){
		const long ActualRead = ov_read(&m_VF, DestPtr, Remain, 0, 2, 1, NULL);
		if(ActualRead<=0){
			m_Terminated = true;
			break;
		}
		Remain -= ActualRead;
		DestPtr += ActualRead;
	}
	if(ReadSize){
		*ReadSize = Size - Remain;
	}
	return(true);
}
//ǂݍ݈ʒuݒ
bool COggReader::Seek(unsigned long Sample){
	if(!IsOpen()){
		DX9AErrOut("Failed : COggReader::Seek Not Open\n");
		return(false);
	}
	if(ov_pcm_seek(&m_VF, Sample)){
		DX9AErrOut("Failed : COggReader::Seek Seek Disable\n");
		return(false);
	}
	m_Terminated = false;
	return(true);
}
//ǂݍ݈ʒu擾
unsigned long COggReader::Tell() const{
	if(!IsOpen()){
		DX9AErrOut("Failed : COggReader::Tell Not Open\n");
		return(0);
	}
	return(static_cast<DWORD>(ov_pcm_tell(const_cast<OggVorbis_File*>(&m_VF))));
}
//ǂݍ݈ʒuI[H
bool COggReader::IsTerminated() const{
	return(m_Terminated);
}

//tH[}bg擾
PCMWAVEFORMAT COggReader::GetFormat() const{
	return(m_PCMWF);
}
//Ŝ̃Tv擾
unsigned long COggReader::GetTotal() const{
	return(m_TotalSamples);
}
//[vC̃Tv擾
unsigned long COggReader::GetLoopIn() const{
	return(m_LoopIn);
}
//[vAEg̃Tv擾
unsigned long COggReader::GetLoopOut() const{
	return(m_LoopOut);
}
//[v񐔂擾
int COggReader::GetLoopCount() const{
	return(m_LoopCount);
}

//[vC̃Tvݒ
void COggReader::SetLoopIn(unsigned long Sample){
	m_LoopIn = Sample;
}
//[vAEg̃Tvݒ
void COggReader::SetLoopOut(unsigned long Sample){
	m_LoopOut = Sample;
}
//[v񐔂ݒ
void COggReader::SetLoopCount(int LoopCount){
	m_LoopCount = LoopCount;
}

//I[v
bool COggReader::Open(const COggFile& File){
	m_File = File;
	//OggJ
	OggVorbis_File VF;
	if(ov_open_callbacks(&m_File, &VF, NULL, 0, COggFile::OggCallbacks)<0){
		m_File.Close();
		DX9AErrOut("Failed : COggReader::Open File Not Found\n");
		return(false);
	}
	//V[N\
	if(!ov_seekable(&VF)){
		ov_clear(&VF);
		DX9AErrOut("Failed : COggReader::Open Seek Disable\n");
		return(false);
	}
	m_VF = VF;
	//W
	const vorbis_info* VI = ov_info(&VF, -1);
	const int SampleBits = 16;
	const int BlockAlign = VI->channels * (SampleBits / 8);
	m_PCMWF.wf.wFormatTag = WAVE_FORMAT_PCM;
	m_PCMWF.wf.nChannels = static_cast<WORD>(VI->channels);
	m_PCMWF.wf.nSamplesPerSec = VI->rate;
	m_PCMWF.wf.nAvgBytesPerSec = VI->rate * BlockAlign;
	m_PCMWF.wf.nBlockAlign = static_cast<WORD>(BlockAlign);
	m_PCMWF.wBitsPerSample = SampleBits;
	m_TotalSamples = static_cast<DWORD>(ov_pcm_total(&VF, -1));
	m_Terminated = false;
	SetLoopIn(0);
	SetLoopOut(GetTotal());
	SetLoopCount(0);
	return(true);
}

#endif

}
}
}